# -*- coding: utf8 -*-
from __future__ import unicode_literals

import driverng

parser_class = "agilent"


def test_decode_result(self):
    worklist = self.decode(
        """
    [Plate Setup]
    Well,Well Type,Dye,Target,Replicate,Sample Name,Well Name
    A1,Standard,FAM,Target,1, ,
    A1,Standard,ROX,ROX,1, ,
    A2,Standard,FAM,Target,1, ,
    A2,Standard,ROX,ROX,1, ,
    A3,Standard,FAM,Target,1, ,
    A3,Standard,ROX,ROX,1, ,

    [Thermal Profile]
    Segment,Plateau,Temperature,Duration,Cycle,DataMarker,TimeStep,TemperatureStep
    RT,1,50,00:10:00,1,No, ,
    Hot Start,1,95,00:03:00,1,No, ,
    Amplification,1,95,00:00:05,40,No, ,
    Amplification,2,60,00:00:10,40,Plateau, ,

    [Tabular Results]
    Well,Well Type,Sample Name,Well Name,Dye,Target,Replicate,R Last,∆R Last,Rn Last,∆Rn Last,RLast/RFirst,Threshold (∆Rn),Baseline Start Cycle,Baseline End Cycle,Cq (∆Rn),Final Call (∆Rn),Quantity (relative),Quantity Avg. (relative),Quantity SD (relative),R² (∆Rn),Slope (∆Rn),Efficiency (%),Cq Avg. Treated Ind. (∆Rn),Cq SD Treated Ind. (∆Rn),∆∆Cq,Rel. Quant. to Norm. (∆Rn),Rel. Quant. to Norm. Avg. (∆Rn),Rel. Quant. to Norm. SD (∆Rn),Rel. Quant. to Norm. Cv(%) (∆Rn),Rel. Quant. to Cal.,Log Fold Change to Cal. (∆Rn),Rel. Quant. to Cal. Avg. (∆Rn),Rel. Quant. to Cal. SD (∆Rn),Rel. Quant. to Cal. Cv(%) (∆Rn),Rel. Quant. to Cal. Avg. - 1 Cq SD (∆Rn),Rel. Quant. to Cal. Avg. + 1 Cq SD (∆Rn),Melt Smoothing #,Melt Norm.,Tm Product 1 (-Rn'(T)),Tm Product 2 (-Rn'(T)),Tm Product 3 (-Rn'(T)),Tm Product 4 (-Rn'(T)),Tm Product 5 (-Rn'(T)),Tm Product 6 (-Rn'(T))
    A1,Standard,1234, ,FAM,Target,1,2994.85073439073,897.488507225894,1.36359703140119,0.408134438322076,1.48676526027916,0.0104235075846078,6,19,20.91,+,10,10,0,0.993,-3.776,84,20.9115898510894,0.0401931578019687, ,N/A,N/A,N/A,0,N/A,N/A,N/A,N/A,0,N/A,N/A,N/A,N/A,No Tm,N/A,N/A,N/A,N/A,N/A
    A2,Standard,88666 , ,FAM,Target,1,3181.92393562295,968.17619120601,1.42885743734493,0.434681945919361,1.48157712842021,0.0104235075846078,7,19,20.87,+,10,10,0,0.993,-3.776,84,20.9115898510894,0.0401931578019687, ,N/A,N/A,N/A,0,N/A,N/A,N/A,N/A,0,N/A,N/A,N/A,N/A,No Tm,N/A,N/A,N/A,N/A,N/A
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "FAMReplicate", "value": "1"},
                    {"code": "FAMR Last", "value": "2994.85073439073"},
                    {"code": "FAMDR Last", "value": "897.488507225894"},
                    {"code": "FAMRn Last", "value": "1.36359703140119"},
                    {"code": "FAMDRn Last", "value": "0.408134438322076"},
                    {"code": "FAMRLast/RFirst", "value": "1.48676526027916"},
                    {"code": "FAMThreshold (DRn)", "value": "0.0104235075846078"},
                    {"code": "FAMBaseline Start Cycle", "value": "6"},
                    {"code": "FAMBaseline End Cycle", "value": "19"},
                    {"code": "FAMCq (DRn)", "value": "20.91"},
                    {"code": "FAMFinal Call (DRn)", "value": "+"},
                    {"code": "FAMQuantity (relative)", "value": "10"},
                    {"code": "FAMQuantity Avg. (relative)", "value": "10"},
                    {"code": "FAMQuantity SD (relative)", "value": "0"},
                    {"code": "FAMR2 (DRn)", "value": "0.993"},
                    {"code": "FAMSlope (DRn)", "value": "-3.776"},
                    {"code": "FAMEfficiency (%)", "value": "84"},
                    {
                        "code": "FAMCq Avg. Treated Ind. (DRn)",
                        "value": "20.9115898510894",
                    },
                    {
                        "code": "FAMCq SD Treated Ind. (DRn)",
                        "value": "0.0401931578019687",
                    },
                    {"code": "FAMDDCq", "value": ""},
                    {"code": "FAMRel. Quant. to Norm. Cv(%) (DRn)", "value": "0"},
                    {"code": "FAMRel. Quant. to Cal. Cv(%) (DRn)", "value": "0"},
                    {"code": "FAMTm Product 1 (-Rn'(T))", "value": "No Tm"},
                ],
                "sample_id": "1234",
                "rack": "A1",
                "type": "patient",
            },
            {
                "results": [
                    {"code": "FAMReplicate", "value": "1"},
                    {"code": "FAMR Last", "value": "3181.92393562295"},
                    {"code": "FAMDR Last", "value": "968.17619120601"},
                    {"code": "FAMRn Last", "value": "1.42885743734493"},
                    {"code": "FAMDRn Last", "value": "0.434681945919361"},
                    {"code": "FAMRLast/RFirst", "value": "1.48157712842021"},
                    {"code": "FAMThreshold (DRn)", "value": "0.0104235075846078"},
                    {"code": "FAMBaseline Start Cycle", "value": "7"},
                    {"code": "FAMBaseline End Cycle", "value": "19"},
                    {"code": "FAMCq (DRn)", "value": "20.87"},
                    {"code": "FAMFinal Call (DRn)", "value": "+"},
                    {"code": "FAMQuantity (relative)", "value": "10"},
                    {"code": "FAMQuantity Avg. (relative)", "value": "10"},
                    {"code": "FAMQuantity SD (relative)", "value": "0"},
                    {"code": "FAMR2 (DRn)", "value": "0.993"},
                    {"code": "FAMSlope (DRn)", "value": "-3.776"},
                    {"code": "FAMEfficiency (%)", "value": "84"},
                    {
                        "code": "FAMCq Avg. Treated Ind. (DRn)",
                        "value": "20.9115898510894",
                    },
                    {
                        "code": "FAMCq SD Treated Ind. (DRn)",
                        "value": "0.0401931578019687",
                    },
                    {"code": "FAMDDCq", "value": ""},
                    {"code": "FAMRel. Quant. to Norm. Cv(%) (DRn)", "value": "0"},
                    {"code": "FAMRel. Quant. to Cal. Cv(%) (DRn)", "value": "0"},
                    {"code": "FAMTm Product 1 (-Rn'(T))", "value": "No Tm"},
                ],
                "sample_id": "88666",
                "rack": "A2",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected


def test_encode_order(self):
    worklist = driverng.Worklist(
        context="order",
        origin="plandeplaque",
        samples=[
            {
                "sample_id": "123456",
                "type": "patient",
                "geometry": {
                    "format_abscissa": "n1",
                    "format_ordinate": "al",
                    "plate_size": "96",
                    "height": 12,
                    "width": 8,
                    "run_protocol": "Protocole kit Variants",
                },
                "wells": [
                    {
                        "abscissa": "1",
                        "barcode": "19E340041469",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E340041496",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E290222772",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E320083207",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E320083237",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {"abscissa": "3", "ordinate": "C", "well_type": 8},
                    {"abscissa": "4", "ordinate": "C", "well_type": 10},
                    {"abscissa": "5", "ordinate": "C", "well_type": 9},
                    {"abscissa": "6", "ordinate": "C", "well_type": 2},
                    {
                        "abscissa": "8",
                        "barcode": "19E290222813",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {"abscissa": "3", "ordinate": "D", "well_type": 8},
                    {"abscissa": "4", "ordinate": "D", "well_type": 10},
                    {"abscissa": "5", "ordinate": "D", "well_type": 9},
                    {"abscissa": "6", "ordinate": "D", "well_type": 2},
                    {"abscissa": "3", "ordinate": "E", "well_type": 8},
                    {"abscissa": "4", "ordinate": "E", "well_type": 10},
                    {"abscissa": "5", "ordinate": "E", "well_type": 9},
                    {"abscissa": "6", "ordinate": "E", "well_type": 2},
                    {
                        "abscissa": "1",
                        "barcode": "19E111106274",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E320083238",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E111106614",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E290222821",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E111106507",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E290222835",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                ],
                "orders": [{"code": "HEX;FAM", "action": "new", "priority": "A"}],
            }
        ],
    )
    message = self.order(worklist=worklist)
    expected = """
    [Plate Setup]
    Well,Well Type,Dye,Target,Replicate,Sample Name,Well Name
    A1,Unknow,HEX,HEX,1,,19E340041469
    A1,Unknow,FAM,FAM,1,,19E340041469
    A8,Unknow,HEX,HEX,8,,19E340041496
    A8,Unknow,FAM,FAM,8,,19E340041496
    B1,Unknow,HEX,HEX,1,,19E290222772
    B1,Unknow,FAM,FAM,1,,19E290222772
    B8,Unknow,HEX,HEX,8,,19E320083207
    B8,Unknow,FAM,FAM,8,,19E320083207
    C1,Unknow,HEX,HEX,1,,19E320083237
    C1,Unknow,FAM,FAM,1,,19E320083237
    C3,Homo Allele A,HEX,HEX,3,,
    C3,Homo Allele A,FAM,FAM,3,,
    C4,Hetero,HEX,HEX,4,,
    C4,Hetero,FAM,FAM,4,,
    C5,Homo Allele B,HEX,HEX,5,,
    C5,Homo Allele B,FAM,FAM,5,,
    C6,NTC,HEX,HEX,6,,
    C6,NTC,FAM,FAM,6,,
    C8,Unknow,HEX,HEX,8,,19E290222813
    C8,Unknow,FAM,FAM,8,,19E290222813
    D3,Homo Allele A,HEX,HEX,3,,
    D3,Homo Allele A,FAM,FAM,3,,
    D4,Hetero,HEX,HEX,4,,
    D4,Hetero,FAM,FAM,4,,
    D5,Homo Allele B,HEX,HEX,5,,
    D5,Homo Allele B,FAM,FAM,5,,
    D6,NTC,HEX,HEX,6,,
    D6,NTC,FAM,FAM,6,,
    E3,Homo Allele A,HEX,HEX,3,,
    E3,Homo Allele A,FAM,FAM,3,,
    E4,Hetero,HEX,HEX,4,,
    E4,Hetero,FAM,FAM,4,,
    E5,Homo Allele B,HEX,HEX,5,,
    E5,Homo Allele B,FAM,FAM,5,,
    E6,NTC,HEX,HEX,6,,
    E6,NTC,FAM,FAM,6,,
    F1,Unknow,HEX,HEX,1,,19E111106274
    F1,Unknow,FAM,FAM,1,,19E111106274
    F8,Unknow,HEX,HEX,8,,19E320083238
    F8,Unknow,FAM,FAM,8,,19E320083238
    G1,Unknow,HEX,HEX,1,,19E111106614
    G1,Unknow,FAM,FAM,1,,19E111106614
    G8,Unknow,HEX,HEX,8,,19E290222821
    G8,Unknow,FAM,FAM,8,,19E290222821
    H1,Unknow,HEX,HEX,1,,19E111106507
    H1,Unknow,FAM,FAM,1,,19E111106507
    H8,Unknow,HEX,HEX,8,,19E290222835
    H8,Unknow,FAM,FAM,8,,19E290222835
    """
    assert message == expected
