# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "arch2000"


def test_qc_preliminaires(self):
    self.configure(archi_dilution_en_alarme=True, archi_lot_non_concatene=False)
    worklist = self.decode(
        """
    H|\\^&|||ARCHITECT^7.00^F3450002654^H1P1O1R1C1Q1L1|||||||P|1|20130207163302
    P|1
    O|1|BIORAD3|BIORAD3^U775^1|^^^30^i B-hCG ST^UNDILUTED^P|R||||||Q||||||||||||||F
    R|1|^^^30^i B-hCG ST^UNDILUTED^P^20904JN00^09111^95370^F|337.00|mIU/mL|316.68 TO 401.92|||F||gwe^gwe||20130207162627|iSR07970
    R|2|^^^30^i B-hCG ST^UNDILUTED^P^20904JN00^09111^95370^P|75127.0000|URL||||F||gwe^gwe||20130207162627|iSR07970
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "BIORAD395370",
                "control_level": "",
                "sample_id": "95370",
                "rack": "U775-1",
                "results": [
                    {
                        "code": "30F",
                        "value": "337.00",
                        "dilution": "",
                        "date": datetime.datetime(2013, 2, 7, 16, 26, 27),
                        "flags": ["dilution: UNDILUTED"],
                        "comments": [],
                        "instrument": "iSR07970",
                        "lots": [
                            {
                                "lot": "20904JN00_09111",
                                "name": "20904JN00_09111",
                                "type": "reagent",
                            }
                        ],
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_abnormal_flags(self):
    date = datetime.datetime(2013, 2, 19, 13, 21, 21)
    worklist = self.decode(
        """
    H|\\^&|||ARCHITECT^7.00^F3450002707^H1P1O1R1C1Q1L1|||||||P|1|20130219132121
    P|1||03060700033030|03060700033030|ANO^NYME||19240812|F
    O|1|303021908901|303021908901^S810^1|^^^1066^c CaC^STANDARD^P|R||||||||||||||||||||F
    R|1|^^^1066^c CaC^STANDARD^P^96511UN12^21947^^F|2.80|mmol/L|2.10 TO 2.55|CNTL^HIGH||F||ADMIN^ADMIN||20130219132121|c802879
    R|2|^^^1067^c CaC^STANDARD^P^96511UN12^21947^^F|3.80|mmol/L|2.10 TO 2.55|HIGH||F||ADMIN^ADMIN||20130219132121|c802879
    R|3|^^^1068^c CaC^STANDARD^P^96511UN12^21947^^F|4.80|mmol/L|2.10 TO 2.55|||F||ADMIN^ADMIN||20130219132121|c802879
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "303021908901",
                "rack": "S810-1",
                "results": [
                    {
                        "code": "1066F",
                        "value": "2.80",
                        "date": date,
                        "flags": ["CNTL", "HIGH"],
                        "comments": [],
                        "dilution": "",
                        "instrument": "c802879",
                        "lots": [
                            {
                                "lot": "96511UN12_21947",
                                "name": "96511UN12_21947",
                                "type": "reagent",
                            }
                        ],
                    },
                    {
                        "code": "1067F",
                        "value": "3.80",
                        "date": date,
                        "flags": ["HIGH"],
                        "dilution": "",
                        "comments": [],
                        "instrument": "c802879",
                        "lots": [
                            {
                                "lot": "96511UN12_21947",
                                "name": "96511UN12_21947",
                                "type": "reagent",
                            }
                        ],
                    },
                    {
                        "code": "1068F",
                        "value": "4.80",
                        "dilution": "",
                        "date": date,
                        "flags": [],
                        "comments": [],
                        "instrument": "c802879",
                        "lots": [
                            {
                                "lot": "96511UN12_21947",
                                "name": "96511UN12_21947",
                                "type": "reagent",
                            }
                        ],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_result_with_dilution(self):
    self.configure(archi_dilution_en_alarme=True)
    worklist = self.decode(
        """
    H|\\^&|||ARCHITECT^9.10^F3457230049^H1P1O1R1C1Q1L1|||||||P|1|20180328184450
    P|1||212604|212604|TEST ABBOTT^DILU BIS||19501212|M
    O|1|101003285410|101003285410^Z665^1|^^^713^Estradiol^1:5^P|R||||||||||||||||||||F
    R|1|^^^713^Estradiol^1:5^P^83036UI00^03744^^F|1425|pg/mL||CNTL||F||AR^AR||20180328184450|iSR05964
    R|2|^^^713^Estradiol^1:5^P^83036UI00^03744^^P|325975|RLU||||F||AR^AR||20180328184450|iSR05964
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "101003285410",
                "rack": "Z665-1",
                "results": [
                    {
                        "code": "713F",
                        "value": "1425",
                        "flags": ["CNTL", "dilution: 1:5"],
                        "comments": [],
                        "dilution": "1:5",
                        "date": datetime.datetime(2018, 3, 28, 18, 44, 50),
                        "instrument": "iSR05964",
                        "lots": [
                            {
                                "lot": "83036UI00_03744",
                                "name": "83036UI00_03744",
                                "type": "reagent",
                            }
                        ],
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_comment_obr(self):
    date = datetime.datetime(2018, 1, 7, 14, 9, 41)
    worklist = self.decode(
        """
        H|\\^&|||ARCHITECT^9.10^F3456310142^H1P1O1R1C1Q1L1|||||||P|1|20180107140941
        P|1|||||||U
        O|1|18500769901|18500769901^S499^4|^^^391^Cyclo^UNDILUTED^P|R||||||||||||||||||||X
        C|1|I|4200 Impossible de lire l'etiquette code-barres dans portoir (S499) position (4).|I
        L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "18500769901",
                "rack": "S499-4",
                "results": [
                    {
                        "code": "COM1",
                        "value": "4200",
                        "date": date,
                        "comments": [
                            "Impossible de lire l'etiquette code-barres dans portoir (S499) position (4)."
                        ],
                        "instrument": None,
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||MCA|||||||P|1
    P|1||036666666||NYME^ANO|NJF|20110513|F|||||||||||||||||^LOC
    O|1|123456||^^^A^^1:1:10\\^^^B||20130603110342|||||A||||||||||||||O
    O|2|123456||^^^C||20130603110342|||||C||||||||||||||O
    L|1
    """
    assert message == expected


def test_controles(self):
    self.configure(archi_lot_non_concatene=False)
    worklist = self.decode(
        """
    H|\\^&|||ARCHITECT^8.10^F5260042031^H1P1O1R1C1Q1L1|||||||P|1|20130826160648
    P|1
    O|1|VITD MOYEN-2|VITD MOYEN-2^R357^1|^^^891^VitD_25OH^UNDILUTED^P|R||||||Q||||||||||||||F
    R|1|^^^891^VitD_25OH^UNDILUTED^P^01013C000^01722^1913A000^F|43.7|ng/mL|38.2 TO 41.8|1-3s||F||VV^VV||20130826150442|iSR03097
    R|2|^^^891^VitD_25OH^UNDILUTED^P^01013C000^01722^1913A000^P|283199|RLU||||F||VV^VV||20130826150442|iSR03097
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "VITD MOYEN-21913A000",
                "control_level": "",
                "sample_id": "1913A000",
                "rack": "R357-1",
                "results": [
                    {
                        "code": "891F",
                        "value": "43.7",
                        "dilution": "",
                        "date": datetime.datetime(2013, 8, 26, 15, 4, 42),
                        "flags": ["1-3s"],
                        "comments": [],
                        "instrument": "iSR03097",
                        "lots": [
                            {
                                "lot": "01013C000_01722",
                                "name": "01013C000_01722",
                                "type": "reagent",
                            }
                        ],
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_query(self):
    worklist = self.decode(
        """
    H|\\^&|||ARCHITECT^8.10^F3450002707^H1P1O1R1C1Q1L1|||||||P|1|20140603105424
    Q|1|^344060308701||^^^ALL||||||||O
    L|1
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "344060308701"}],
    }
    assert worklist == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            H|\\^&|||ARCHITECT^8.10^F3450002707^H1P1O1R1C1Q1L1|||||||P|1|20140603105424
            Q|1|^344060308701||^^^ALL||||||||O
            L|1"""
            )
        },
    )
    expected = """
    H|\\^&|||MCA|||||||P|1
    Q|1|^344060308701||^^^ALL||||||||X
    L|1
    """
    assert message == expected


def test_qc_error_without_result(self):
    worklist = self.decode(
        """
    H|\\^&|||ARCHITECT^8.10^Number1^H1P1O1R1C1Q1L1|||||||P|1|20171117111037
    P|1
    O|1|Multiqual L1|Multiqual L1^G269^1|^^^1029^c Lipa^STANDARD^P|R||||||Q||||||||||||||X
    C|1|I|1054 Impossible de calculer le rsultat, erreur lors de la vrification de la raction.|I
    L|1
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected
