# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "starlet"


def test_decode_res(self):
    date = datetime.datetime(2019, 1, 7, 9, 46, 4, 540000)
    worklist = self.decode(
        """
    MSH|^~\\&|Seegene Viewer|Seegene|||20190107094604.54+0100||ORU^R01^ORU_R01|64801|P|2.5
    ORC|NW|||||||||FVE35
    OBR|1||883004019139^F03|Allplex GI-Bacteria (I) Assay (96 plate)^-
    NTE|1|P|FAM~Sh/EI~~-~Ct.~N/A
    NTE|2|P|FAM~Cam~~-~Ct.~N/A
    NTE|3|P|HEX~Yer~~-~Ct.~N/A
    NTE|4|P|Cal Red 610~Vib~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~CdB~~-~Ct.~N/A
    NTE|6|P|Quasar 670~Aer~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Sal~~-~Ct.~N/A
    NTE|8|P|HEX~IC~~+~Ct.~35.11
    OBX|1|ST|PCR LOT
    NTE|1||PCR Lot~SDF122K04
    OBX|2|ST|Extraction Lot
    NTE|1||Extraction Lot~UC9622K37
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "OPERATEUR", "value": "FVE35"},
                    {
                        "code": "POSITION_CFX",
                        "date": date,
                        "operator": "FVE35",
                        "value": "F03",
                    },
                    {"code": "Sh/EI", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Cam", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Yer", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Vib", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "CdB", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Aer", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Sal", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "IC", "date": date, "operator": "FVE35", "value": "+"},
                    {
                        "code": "LEVEL_IC",
                        "date": date,
                        "operator": "FVE35",
                        "value": "35.11",
                    },
                    {
                        "code": "PCR Lot",
                        "date": date,
                        "operator": "FVE35",
                        "value": "SDF122K04",
                    },
                    {
                        "code": "Extraction Lot",
                        "date": date,
                        "operator": "FVE35",
                        "value": "UC9622K37",
                    },
                ],
                "sample_id": "883004019139",
                "rack": "F03",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_other_result(self):
    date = datetime.datetime(2021, 2, 10, 22, 45, 42)
    worklist = self.decode(
        """
    MSH|^~\\&|SARS-CoV-2 Viewer|Seegene|||20210210224542+0100||ORU^R01^ORU_R01|208701|P|2.5
    OBR|1||0000077556^F03|Allplexâ¢ SARS-CoV-2 Assay^-
    NTE|1|P|FAM~E gene~~-~Ct.~N/A
    NTE|2|P|Cal Red 610~RdRP/S gene~~-~Ct.~N/A
    NTE|3|P|Quasar 670~N gene~~-~Ct.~N/A
    NTE|4|P|HEX~IC~~+~Ct.~24.84
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "CONCLUSION_GENERALE", "date": date, "value": "-"},
                    {"code": "POSITION_CFX", "date": date, "value": "F03"},
                    {"code": "E gene", "date": date, "value": "-"},
                    {"code": "RdRP/S gene", "date": date, "value": "-"},
                    {"code": "N gene", "date": date, "value": "-"},
                    {"code": "IC", "date": date, "value": "+"},
                    {"code": "LEVEL_IC", "date": date, "value": "24.84"},
                ],
                "sample_id": "0000077556",
                "rack": "F03",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_revision1(self):
    date = datetime.datetime(2017, 10, 12, 13, 29, 0, 586000)
    worklist = self.decode(
        """
    MSH|^~\\&|Seegene Viewer|Seegene|||20171012132900.586+0900||ORU^R01^ORU_R01|1101|P|2.5
    OBR|1||15027611^A01|Anyplex™ II STI-5 Detection(96 plate)^-
    NTE|1|P|FAM~UU~CODE1~-~Result~N/A
    NTE|2|P|FAM~UP~~-~Result~N/A
    NTE|3|P|HEX~MG~Meningitis~-~Result~N/A
    NTE|4|P|HEX~MH~CODE2~-~Result~N/A
    NTE|5|P|Cal Red 610~TV~~-~Result~N/A
    NTE|6|P|Quasar 670~IC~~++~Result~194.57
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A01",
                "results": [
                    {"code": "CONCLUSION_GENERALE", "date": date, "value": "-"},
                    {"code": "POSITION_CFX", "date": date, "value": "A01"},
                    {"code": "CODE1", "date": date, "value": "-"},
                    {"code": "UP", "date": date, "value": "-"},
                    {"code": "Meningitis", "date": date, "value": "-"},
                    {"code": "CODE2", "date": date, "value": "-"},
                    {"code": "TV", "date": date, "value": "-"},
                    {"code": "IC", "date": date, "value": "++"},
                    {"code": "LEVEL_IC", "date": date, "value": "194.57"},
                ],
                "sample_id": "15027611",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2018, 9, 6, 17, 35, 20, 190000)
    worklist = self.decode(
        """
    MSH|^~\\&|Seegene Viewer|Seegene|||20180906173520.19+0100||ORU^R01^ORU_R01|23401|P|2.5
    OBR|1||297972^A01|Allplex™ Respiratory Panel 1 (96 plate)^-
    NTE|1|P|FAM~RSV A~~-~Ct.~N/A
    NTE|2|P|FAM~Flu A~~-~Ct.~N/A
    NTE|3|P|HEX~RSV B~~-~Ct.~N/A
    NTE|4|P|HEX~Flu B~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~Flu A-H1pdm09~~-~Ct.~N/A
    NTE|6|P|Cal Red 610~Flu A-H1~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Flu A-H3~~-~Ct.~N/A
    NTE|8|P|Quasar 670~IC~~+~Ct.~27.21
    OBX|1|ST|PCR LOT
    NTE|1||PCR Lot~GI0418E18012102
    OBX|2|ST|Extraction Lot
    NTE|1||Extraction Lot~UC9618J26
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "CONCLUSION_GENERALE", "value": "-", "date": date},
                    {"code": "POSITION_CFX", "date": date, "value": "A01"},
                    {"code": "RSV A", "value": "-", "date": date},
                    {"code": "Flu A", "value": "-", "date": date},
                    {"code": "RSV B", "value": "-", "date": date},
                    {"code": "Flu B", "value": "-", "date": date},
                    {"code": "Flu A-H1pdm09", "value": "-", "date": date},
                    {"code": "Flu A-H1", "value": "-", "date": date},
                    {"code": "Flu A-H3", "value": "-", "date": date},
                    {"code": "IC", "value": "+", "date": date},
                    {"code": "LEVEL_IC", "value": "27.21", "date": date},
                ],
                "sample_id": "297972",
                "rack": "A01",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_cfx_with_pid(self):
    date = datetime.datetime(2019, 3, 5, 18, 47, 42, 291000)
    worklist = self.decode(
        """
    MSH|^~\\&|Seegene Viewer|Seegene|||20190305184742.291+0100||ORU^R01^ORU_R01|57001|P|2.5
    PID|||TEST ANONYME
    OBR|1||9203057240^A01|Allplex GI-Bacteria (I) Assay (96 plate)^-
    NTE|1|P|FAM~Sh/EI~~-~Ct.~N/A
    NTE|2|P|FAM~Cam~~-~Ct.~N/A
    NTE|3|P|HEX~Yer~~-~Ct.~N/A
    NTE|4|P|Cal Red 610~Vib~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~CdB~~-~Ct.~N/A
    NTE|6|P|Quasar 670~Aer~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Sal~~-~Ct.~N/A
    NTE|8|P|HEX~IC~~+~Ct.~33.95
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CONCLUSION_GENERALE",
                        "value": "-",
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "POSITION_CFX",
                        "date": date,
                        "value": "A01",
                        "operator": "",
                    },
                    {"code": "Sh/EI", "value": "-", "date": date, "operator": ""},
                    {"code": "Cam", "value": "-", "date": date, "operator": ""},
                    {"code": "Yer", "value": "-", "date": date, "operator": ""},
                    {"code": "Vib", "value": "-", "date": date, "operator": ""},
                    {"code": "CdB", "value": "-", "date": date, "operator": ""},
                    {"code": "Aer", "value": "-", "date": date, "operator": ""},
                    {"code": "Sal", "value": "-", "date": date, "operator": ""},
                    {"code": "IC", "value": "+", "date": date, "operator": ""},
                    {
                        "code": "LEVEL_IC",
                        "value": "33.95",
                        "date": date,
                        "operator": "",
                    },
                ],
                "rack": "A01",
                "sample_id": "9203057240",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_cfx(self):
    date = datetime.datetime(2018, 12, 5, 16, 41, 9, 673000)
    worklist = self.decode(
        """
    MSH|^~\\&|Seegene Viewer|Seegene|||20181205164109.673+0100||ORU^R01^ORU_R01|54201|P|2.5
    PID|||TEST ANONYME
    ORC|NW|||||||||FVE35
    OBR|1||024903128073^A01|Allplex GI Assay (96 plate) : Allplexâ¢ GI-Bacteria (I) Assay (96 plate)^Sh/EI
    NTE|1|P|FAM~Sh/EI~~+~Ct.~26.23
    NTE|2|P|FAM~Cam~~-~Ct.~N/A
    NTE|3|P|HEX~Yer~~-~Ct.~N/A
    NTE|4|P|Cal Red 610~Vib~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~CdB~~-~Ct.~N/A
    NTE|6|P|Quasar 670~Aer~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Sal~~-~Ct.~N/A
    NTE|8|P|HEX~IC~~+~Ct.~35.7
    OBX|1|ST|PCR LOT
    NTE|1||PCR Lot~SDF122K04
    NTE|2||PCR Lot~SDF122J03
    OBX|2|ST|Extraction Lot
    NTE|1||Extraction Lot~UC9622K37
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A01",
                "results": [
                    {"code": "OPERATEUR", "value": "FVE35"},
                    {
                        "code": "CONCLUSION_GENERALE",
                        "date": date,
                        "operator": "FVE35",
                        "value": "Sh/EI",
                    },
                    {
                        "code": "POSITION_CFX",
                        "date": date,
                        "operator": "FVE35",
                        "value": "A01",
                    },
                    {"code": "Sh/EI", "date": date, "operator": "FVE35", "value": "+"},
                    {
                        "code": "LEVEL_Sh/EI",
                        "date": date,
                        "operator": "FVE35",
                        "value": "26.23",
                    },
                    {"code": "Cam", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Yer", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Vib", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "CdB", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Aer", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "Sal", "date": date, "operator": "FVE35", "value": "-"},
                    {"code": "IC", "date": date, "operator": "FVE35", "value": "+"},
                    {
                        "code": "LEVEL_IC",
                        "date": date,
                        "operator": "FVE35",
                        "value": "35.7",
                    },
                    {
                        "code": "PCR Lot",
                        "date": date,
                        "operator": "FVE35",
                        "value": "SDF122K04;SDF122J03",
                    },
                    {
                        "code": "Extraction Lot",
                        "date": date,
                        "operator": "FVE35",
                        "value": "UC9622K37",
                    },
                ],
                "sample_id": "024903128073",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_resultat_avec_conclusion_generale(self):
    date = datetime.datetime(2020, 3, 16, 14, 58, 0, 827000)
    worklist = self.decode(
        """
    MSH|^~\\&|2019-nCoV Viewer|Seegene|||20200316145800.827+0100||ORU^R01^ORU_R01|201|P|2.5
    OBR|1||19E180028150^C01|Allplexâ„¢ 2019-nCoV Assay^Invalid
    NTE|1|P|FAM~E gene~~-~Ct.~N/A
    NTE|2|P|Cal Red 610~RdRP gene~~-~Ct.~N/A
    NTE|3|P|Quasar 670~N gene~~-~Ct.~N/A
    NTE|4|P|HEX~IC~~-~Ct.~N/A
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "CONCLUSION_GENERALE", "value": "Invalid", "date": date},
                    {"code": "POSITION_CFX", "date": date, "value": "C01"},
                    {"code": "E gene", "value": "-", "date": date},
                    {"code": "RdRP gene", "value": "-", "date": date},
                    {"code": "N gene", "value": "-", "date": date},
                    {"code": "IC", "value": "-", "date": date},
                ],
                "sample_id": "19E180028150",
                "rack": "C01",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_option_include_all(self):
    date = datetime.datetime(2020, 9, 21, 11, 4, 40, 972000)
    self.configure(starlet_include_results_na=True)
    worklist = self.decode(
        """
    MSH|^~\\&|SARS-CoV-2 Viewer|Seegene|||20200921110440.972+0200||ORU^R01^ORU_R01|4001|P|2.5
    OBR|1||751091411183^A01|Allplexâ¢ SARS-CoV-2 Assay^SARS-CoV-2
    NTE|1|P|FAM~E gene~~+~Ct.~N/A
    NTE|2|P|Cal Red 610~RdRP/S gene~~+~Ct.~24.16
    NTE|3|P|Quasar 670~N gene~~+~Ct.~21.09
    NTE|4|P|HEX~IC~~+~Ct.~25.44
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CONCLUSION_GENERALE",
                        "value": "SARS-CoV-2",
                        "date": date,
                    },
                    {"code": "POSITION_CFX", "date": date, "value": "A01"},
                    {"code": "E gene", "value": "+", "date": date},
                    {"code": "LEVEL_E gene", "value": "N/A", "date": date},
                    {"code": "RdRP/S gene", "value": "+", "date": date},
                    {"code": "LEVEL_RdRP/S gene", "value": "24.16", "date": date},
                    {"code": "N gene", "value": "+", "date": date},
                    {"code": "LEVEL_N gene", "value": "21.09", "date": date},
                    {"code": "IC", "value": "+", "date": date},
                    {"code": "LEVEL_IC", "value": "25.44", "date": date},
                ],
                "sample_id": "751091411183",
                "rack": "A01",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2018, 2, 12, 18, 6, 36, 198000)
    worklist = self.decode(
        """
    MSH|^~\\&|Seegene Viewer|Seegene|||20180212180636.198+0100||ORU^R01^ORU_R01|28401|P|2.5
    OBR|1||PC^A09|Allplex™ Respiratory Panel 3 (96 plate)^Positive Control(+)
    NTE|1|P|FAM~OC43~~+~Ct.~21.53
    NTE|2|P|FAM~HBoV~~+~Ct.~22.13
    NTE|3|P|HEX~229E~~+~Ct.~22.79
    NTE|4|P|HEX~NL63~~+~Ct.~21.34
    NTE|5|P|Cal Red 610~HRV~~+~Ct.~23.50
    NTE|6|P|Quasar 670~IC~~+~Ct.~18.80
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CONCLUSION_GENERALE",
                        "value": "Positive Control(+)",
                        "date": date,
                    },
                    {"code": "OC43", "value": "+", "date": date},
                    {"code": "LEVEL_OC43", "value": "21.53", "date": date},
                    {"code": "HBoV", "value": "+", "date": date},
                    {"code": "LEVEL_HBoV", "value": "22.13", "date": date},
                    {"code": "229E", "value": "+", "date": date},
                    {"code": "LEVEL_229E", "value": "22.79", "date": date},
                    {"code": "NL63", "value": "+", "date": date},
                    {"code": "LEVEL_NL63", "value": "21.34", "date": date},
                    {"code": "HRV", "value": "+", "date": date},
                    {"code": "LEVEL_HRV", "value": "23.50", "date": date},
                    {"code": "IC", "value": "+", "date": date},
                    {"code": "LEVEL_IC", "value": "18.80", "date": date},
                ],
                "sample_id": "PC",
                "control_lot": "PC",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    message = self.order(
        worklist=worklist,
        context={"control_id": "1521651964637", "placer_order_number": "1234547"},
    )
    expected = """
    MSH|^~\\&|MCA|MCA|Seegene Viewer|Seegene|20130715141912||ORU^R30|||2.5
    PID||123456|||ANO^NYME
    ORC|NW
    OBR||||123456
    NTE|||A~B
    """
    assert message == expected


def test_decode_all_negative_obr_results(self):
    date = datetime.datetime(2020, 10, 23, 10, 5, 6, 827000)
    worklist = self.decode(
        """
    MSH|^~\\&|SARS-CoV-2 Viewer|Seegene|||20201023100506.827+0200||ORU^R01^ORU_R01|201|P|2.5
    OBR|1||200000288401 VRS 1/2^A03|Allplex™ SARS-CoV-2/FluA/FluB/RSV Assay^
    NTE|1|P|FAM~S gene~~-~Ct.~N/A
    NTE|2|P|FAM~RSV~~+~Ct.~15.67
    NTE|3|P|HEX~RdRP gene~~-~Ct.~N/A
    NTE|4|P|HEX~Flu B~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~N gene~~-~Ct.~N/A
    NTE|6|P|Cal Red 610~Flu A~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Endo IC~~+~Ct.~21.36
    NTE|8|P|Quasar 670~Exo IC~~+~Ct.~21.77
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A03",
                "results": [
                    {"code": "POSITION_CFX", "date": date, "value": "A03"},
                    {"code": "SARS-CoV-2OBR", "date": date, "value": "Négatif"},
                    {"code": "FluAOBR", "date": date, "value": "Négatif"},
                    {"code": "FluBOBR", "date": date, "value": "Négatif"},
                    {"code": "RSVOBR", "date": date, "value": "Négatif"},
                    {"code": "S gene", "date": date, "value": "-"},
                    {"code": "RSV", "date": date, "value": "+"},
                    {"code": "LEVEL_RSV", "date": date, "value": "15.67"},
                    {"code": "RdRP gene", "date": date, "value": "-"},
                    {"code": "Flu B", "date": date, "value": "-"},
                    {"code": "N gene", "date": date, "value": "-"},
                    {"code": "Flu A", "date": date, "value": "-"},
                    {"code": "Endo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Endo IC", "date": date, "value": "21.36"},
                    {"code": "Exo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Exo IC", "date": date, "value": "21.77"},
                ],
                "sample_id": "200000288401 VRS 1/2",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_all_positive_obr_results(self):
    date = datetime.datetime(2020, 10, 23, 10, 5, 6, 843000)
    worklist = self.decode(
        """
    MSH|^~\\&|SARS-CoV-2 Viewer|Seegene|||20201023100506.843+0200||ORU^R01^ORU_R01|301|P|2.5
    OBR|1||700000041773 CoVID pur^A04|Allplex™ SARS-CoV-2/FluA/FluB/RSV Assay^SARS-CoV-2, Flu A, Flu B, RSV
    NTE|1|P|FAM~S gene~~+~Ct.~13.21
    NTE|2|P|FAM~RSV~~-~Ct.~N/A
    NTE|3|P|HEX~RdRP gene~~+~Ct.~13.72
    NTE|4|P|HEX~Flu B~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~N gene~~+~Ct.~14.51
    NTE|6|P|Cal Red 610~Flu A~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Endo IC~~+~Ct.~23.66
    NTE|8|P|Quasar 670~Exo IC~~+~Ct.~21.93
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A04",
                "results": [
                    {
                        "code": "CONCLUSION_GENERALE",
                        "date": date,
                        "value": "SARS-CoV-2, Flu A, Flu B, RSV",
                    },
                    {"code": "POSITION_CFX", "date": date, "value": "A04"},
                    {"code": "SARS-CoV-2OBR", "date": date, "value": "Positif"},
                    {"code": "FluAOBR", "date": date, "value": "Positif"},
                    {"code": "FluBOBR", "date": date, "value": "Positif"},
                    {"code": "RSVOBR", "date": date, "value": "Positif"},
                    {"code": "S gene", "date": date, "value": "+"},
                    {"code": "LEVEL_S gene", "date": date, "value": "13.21"},
                    {"code": "RSV", "date": date, "value": "-"},
                    {"code": "RdRP gene", "date": date, "value": "+"},
                    {"code": "LEVEL_RdRP gene", "date": date, "value": "13.72"},
                    {"code": "Flu B", "date": date, "value": "-"},
                    {"code": "N gene", "date": date, "value": "+"},
                    {"code": "LEVEL_N gene", "date": date, "value": "14.51"},
                    {"code": "Flu A", "date": date, "value": "-"},
                    {"code": "Endo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Endo IC", "date": date, "value": "23.66"},
                    {"code": "Exo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Exo IC", "date": date, "value": "21.93"},
                ],
                "sample_id": "700000041773 CoVID pur",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_positive_and_negative_obr_results(self):
    date = datetime.datetime(2020, 10, 23, 10, 5, 6, 890000)
    worklist = self.decode(
        """
    MSH|^~\\&|SARS-CoV-2 Viewer|Seegene|||20201023100506.89+0200||ORU^R01^ORU_R01|801|P|2.5
    OBR|1||Mélange FluA/FluB^A09|Allplex™ SARS-CoV-2/FluA/FluB/RSV Assay^Flu B, Flu A
    NTE|1|P|FAM~S gene~~-~Ct.~N/A
    NTE|2|P|FAM~RSV~~-~Ct.~N/A
    NTE|3|P|HEX~RdRP gene~~-~Ct.~N/A
    NTE|4|P|HEX~Flu B~~+~Ct.~14.82
    NTE|5|P|Cal Red 610~N gene~~-~Ct.~N/A
    NTE|6|P|Cal Red 610~Flu A~~+~Ct.~18.48
    NTE|7|P|Quasar 670~Endo IC~~+~Ct.~20.90
    NTE|8|P|Quasar 670~Exo IC~~+~Ct.~22.08
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A09",
                "results": [
                    {
                        "code": "CONCLUSION_GENERALE",
                        "date": date,
                        "value": "Flu B, Flu A",
                    },
                    {"code": "POSITION_CFX", "date": date, "value": "A09"},
                    {"code": "SARS-CoV-2OBR", "date": date, "value": "Négatif"},
                    {"code": "FluAOBR", "date": date, "value": "Positif"},
                    {"code": "FluBOBR", "date": date, "value": "Positif"},
                    {"code": "RSVOBR", "date": date, "value": "Négatif"},
                    {"code": "S gene", "date": date, "value": "-"},
                    {"code": "RSV", "date": date, "value": "-"},
                    {"code": "RdRP gene", "date": date, "value": "-"},
                    {"code": "Flu B", "date": date, "value": "+"},
                    {"code": "LEVEL_Flu B", "date": date, "value": "14.82"},
                    {"code": "N gene", "date": date, "value": "-"},
                    {"code": "Flu A", "date": date, "value": "+"},
                    {"code": "LEVEL_Flu A", "date": date, "value": "18.48"},
                    {"code": "Endo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Endo IC", "date": date, "value": "20.90"},
                    {"code": "Exo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Exo IC", "date": date, "value": "22.08"},
                ],
                "sample_id": "Mélange FluA/FluB",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_invalid_obr_results(self):
    date = datetime.datetime(2020, 10, 23, 10, 5, 6, 890000)
    worklist = self.decode(
        """
    MSH|^~\\&|SARS-CoV-2 Viewer|Seegene|||20201023100506.89+0200||ORU^R01^ORU_R01|901|P|2.5
    OBR|1||T lyse chinois sans CoVID^A10|Allplex™ SARS-CoV-2/FluA/FluB/RSV Assay^Invalid 1)
    NTE|1|P|FAM~S gene~~-~Ct.~N/A
    NTE|2|P|FAM~RSV~~-~Ct.~N/A
    NTE|3|P|HEX~RdRP gene~~-~Ct.~N/A
    NTE|4|P|HEX~Flu B~~-~Ct.~N/A
    NTE|5|P|Cal Red 610~N gene~~-~Ct.~N/A
    NTE|6|P|Cal Red 610~Flu A~~-~Ct.~N/A
    NTE|7|P|Quasar 670~Endo IC~~-~Ct.~N/A
    NTE|8|P|Quasar 670~Exo IC~~+~Ct.~21.93
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A10",
                "results": [
                    {
                        "code": "CONCLUSION_GENERALE",
                        "date": date,
                        "value": "Invalid 1)",
                    },
                    {"code": "POSITION_CFX", "date": date, "value": "A10"},
                    {"code": "SARS-CoV-2OBR", "date": date, "value": "Invalide"},
                    {"code": "FluAOBR", "date": date, "value": "Invalide"},
                    {"code": "FluBOBR", "date": date, "value": "Invalide"},
                    {"code": "RSVOBR", "date": date, "value": "Invalide"},
                    {"code": "S gene", "date": date, "value": "-"},
                    {"code": "RSV", "date": date, "value": "-"},
                    {"code": "RdRP gene", "date": date, "value": "-"},
                    {"code": "Flu B", "date": date, "value": "-"},
                    {"code": "N gene", "date": date, "value": "-"},
                    {"code": "Flu A", "date": date, "value": "-"},
                    {"code": "Endo IC", "date": date, "value": "-"},
                    {"code": "Exo IC", "date": date, "value": "+"},
                    {"code": "LEVEL_Exo IC", "date": date, "value": "21.93"},
                ],
                "sample_id": "T lyse chinois sans CoVID",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
