# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

import driverng

parser_class = "sysmex_suit"

MAPPING_CLASS = "sysmex_suit"
ANALYSEUR_RACK_DATA = """
H|^~\\&|||||||||||A.2|201303211057
P|1|0031900944903|0031900944903|0031900944903|ANO^NYME||19710214|M
OBR|1||04140076908|smear~h_inst~h_tube~h_rack|R||20130321||||L
OBX|1|NM|SMEAR||0||||||F^|||||
OBX|2|NM|H_INST||LASC||||||F^|||||
OBX|3|NM|H_TUBE||04||||||F^|||||
OBX|4|NM|H_RACK||100077||||||F^|||||
OBX|2|NM|H_INID||ID||||||F^|||||
L|1||2|44"""


def test_get_queries(self):
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201206251820
Q|1||1125017208|||201206251820
L|1||0|3"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "1125017208"}],
    }
    assert result == expected


def test_no_order_query(self):
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """H|^~\\&|||||||||||A.2|201206251820
Q|1||1125017208|||201206251820
L|1||0|3"""
            )
        },
    )
    expected = """H|^~\\&|||||||||||A.2|201307151419
        P|1
        OBR|1|1125017208
        L|1||1|4"""
    assert result == expected


def test_no_order_query_uf500(self):
    self.configure(uf500_tube_inconnu=True)
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """H|^~\\&|||||||||||A.2|201206251820
Q|1||1125017208|||201206251820
L|1||0|3"""
            )
        },
    )
    expected = """H|^~\\&|||||||||||A.2|201307151419
P|1
OBR|1|1125017208||RBC~XTAL~EC~BACT~WBC
L|1||1|4"""
    assert result == expected


def test_analyseur_format(self):
    klass = driverng.get(MAPPING_CLASS)
    for parser, expected in (
        (klass(sysmex_suit_analyseur_format="INST"), "LASC"),
        (klass(sysmex_suit_analyseur_format="INID"), "ID"),
    ):
        results = parser.get_result_dict(parser.parse(ANALYSEUR_RACK_DATA))
        assert results["04140076908"]["resultats"][0]["analyseur"] == expected


def test_rack_format(self):
    klass = driverng.get(MAPPING_CLASS)
    for parser, expected in (
        (klass(), "100077-004-LASC-ID"),
        (
            klass(sysmex_suit_rack_format="%(RACK)s-%(TUBE)s-%(INST)s-%(INID)s"),
            "100077-004-LASC-ID",
        ),
        (klass(sysmex_suit_rack_format="%(RACK)s-%(TUBE)s"), "100077-004"),
    ):
        results = parser.get_result_dict(parser.parse(ANALYSEUR_RACK_DATA))
        assert results["04140076908"]["rack"] == expected


def test_graphe_png_multi(self):
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201303201049
    P|1
    OBR|1||303032001121|wbc|||201303201040
    P|2
    OBR|1||303032000921|wbc|||201303201042
    OBX|1|NM|WBC||6.78^^1|10*3/uL|||||N^|201303201042||||
    C|1||PNG&R&20130320&R&2013_03_20_10_42_303032000921_RBC_FSC.PNG
    C|3||PNG&R&20130320&R&2013_03_20_10_42_303032000921_PLT.PNG
    C|10||PNG&R&20130320&R&2013_03_20_10_40_303032001121_RBC_FSC.PNG
    C|12||PNG&R&20130320&R&2013_03_20_10_40_303032001121_PLT.PNG
    L|1||2"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "303032000921",
                "rack": "---",
                "results": [
                    {
                        "code": "RBC_FSC.PNG",
                        "value": "PNG/20130320/2013_03_20_10_42_303032000921_RBC_FSC.PNG",
                    },
                    {
                        "code": "PLT.PNG",
                        "value": "PNG/20130320/2013_03_20_10_42_303032000921_PLT.PNG",
                    },
                ],
            },
            {
                "type": "patient",
                "sample_id": "303032001121",
                "rack": "---",
                "results": [
                    {
                        "code": "RBC_FSC.PNG",
                        "value": "PNG/20130320/2013_03_20_10_40_303032001121_RBC_FSC.PNG",
                    },
                    {
                        "code": "PLT.PNG",
                        "value": "PNG/20130320/2013_03_20_10_40_303032001121_PLT.PNG",
                    },
                ],
            },
        ],
    }
    assert result == expected


def test_flag_obx_27826(self):
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201305301144
    P|1|||||||U||||||||||||||||||||||||
    OBR|1||1005300087||||201305301144|||||||||||||||201305301144||002|||
    OBX|13|NM|FLAG_BACT||+||||||F^|||||
    OBX|18|NM|BACT||05170.50|/uL|||||F^|||||
    OBX|19|NM|FLAG_XTAL||+||||||F^|||||
    OBX|23|NM|XTAL||00069.00|/uL||L|||F^|||||
    OBX|24|NM|YLC||00000.00|/uL|||||F^|||||
    OBX|30|NM|H_RACK||2||||||F^|||||
    OBX|31|NM|H_TUBE||9||||||F^|||||
    OBX|32|ST|H_INID||12037||||||F^|||||
    OBX|33|ST|H_INST||UF-500i||||||F^|||||
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "1005300087",
                "rack": "2-009-UF-500i-12037",
                "results": [
                    {
                        "flags": [],
                        "instrument": "UF-500i",
                        "date": None,
                        "value": "+",
                        "code": "FLAG_BACT",
                    },
                    {
                        "flags": ["+"],
                        "instrument": "UF-500i",
                        "date": None,
                        "value": "05170.50",
                        "code": "BACT",
                    },
                    {
                        "flags": [],
                        "instrument": "UF-500i",
                        "date": None,
                        "value": "+",
                        "code": "FLAG_XTAL",
                    },
                    {
                        "flags": ["L", "+"],
                        "instrument": "UF-500i",
                        "date": None,
                        "value": "00069.00",
                        "code": "XTAL",
                    },
                    {
                        "flags": [],
                        "instrument": "UF-500i",
                        "date": None,
                        "value": "00000.00",
                        "code": "YLC",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_resu_comment(self):
    self.configure(traiter_comment_obx=True)
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201306041131
    P|3|00605642624|00605642624|00605642624|ANO^NYME||19490325|M
    OBR|1||04130124536||A||20130604||||L|||20130604||||||||20130604||001|F
    OBX|6|NM|HGB^HGB|D|7.5^XN_TEL|g/dL|||||F^Validate|202202231411|||XN-10^40876^333^3|0134410a
     L|1||4|56"""
    )
    assert result.samples[0].results[0].comments == ["XN_TEL"]


def test_decode_alarme_optique(self):
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201306041131
    P|3|00605642624|00605642624|00605642624|ANO^NYME||19490325|M
    OBR|1||04130124536||A||20130604||||L|||20130604||||||||20130604||001|F
    OBX|8|NM|PLT||30^W^1|10*3/uL||W|||F^|201306041128
    L|1||4|56"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "04130124536",
                "type": "patient",
                "rack": "---",
                "results": [
                    {
                        "code": "PLT",
                        "value": "30",
                        "flags": ["W", "O"],
                        "date": datetime.datetime(2013, 6, 4, 11, 28),
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_analyseur(self):
    result = self.decode(
        """
    H|^~\\&|||||||||||A.2|201310031049
    S|24|Manual|12037|||QC||||QC18|COND.|00035.00|||20131003104905|
    S|24|Open|12037|||QC||||QC18|BLI|42.00|||20131003104905|
    S|28|Manual|12037|||QC||||QC18|H_RACK||||20131003104905|
    S|29|Manual|12037|||QC||||QC18|H_TUBE||||20131003104905|
    S|30|Manual|12037|||QC||||QC18|H_INID|12037|||20131003104905|
    S|31|Manual|12037|||QC||||QC18|H_INST|UF-500i|||20131003104905|
    L|1||0|33"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_lot": "QC18",
                "results": [
                    {
                        "instrument": "UF-500i",
                        "date": datetime.datetime(2013, 10, 3, 10, 49, 5),
                        "value": "00035.00",
                        "code": "COND.",
                    },
                    {
                        "instrument": "UF-500i",
                        "date": datetime.datetime(2013, 10, 3, 10, 49, 5),
                        "value": "42.00",
                        "code": "BLI",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_qc_sans_analyseur(self):
    result = self.decode(
        """
    H|^~\\&|||||||||||A.2|201310041026
    S|1|Manual|XT2000|||QC||||QC-23090801|WBC|3.94|||201310041025|
    S|1|Open|XT2000|||QC||||QC-23090801|RBC|42.94|||201310041025|
    L|1||0|41"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_level": "0801",
                "control_lot": "23090801",
                "control_name": "2309",
                "results": [
                    {
                        "instrument": "XT2000-OPEN",
                        "date": datetime.datetime(2013, 10, 4, 10, 25),
                        "value": "3.94",
                        "code": "WBC",
                    },
                    {
                        "instrument": "XT2000",
                        "date": datetime.datetime(2013, 10, 4, 10, 25),
                        "value": "42.94",
                        "code": "RBC",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|^~\\&|||||||||||A.2|201307151419
        P|1|036666666|036666666||ANO^NYME|NJF|20110513|F|||||ATTEND^LAGRANGE Joseph-Loui||||||||||||SER
        OBR|1|123456||A~B|S||201305131142||||L||Commentaire patient no1
        L|1||1|4"""
    assert result == expected


def test_order_inverser_nom_prenom(self):
    self.configure(inverser_nom_prenom=True)
    result = self.order()
    expected = """H|^~\\&|||||||||||A.2|201307151419
        P|1|036666666|036666666||NYME^ANO|NJF|20110513|F|||||ATTEND^LAGRANGE Joseph-Loui||||||||||||SER
        OBR|1|123456||A~B|S||201305131142||||L||Commentaire patient no1
        L|1||1|4"""
    assert result == expected


def test_no_legacy_mode_open_sans_analyseur(self):
    self.configure(legacy_mode_open=False)
    date = datetime.datetime(2013, 10, 4, 10, 25)
    result = self.decode(
        """
    H|^~\\&|||||||||||A.2|201310041026
    S|1|Manual|XT2000|||QC||||QC-23090801|WBC|3.94|||201310041025|
    S|1|Open|XT2000|||QC||||QC-23090801|RBC|42.94|||201310041025|
    S|1|Unknown|XT2000|||QC||||QC-23090801|PLT|42|||201310041025|
    L|1||0|41"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_level": "0801",
                "control_lot": "23090801",
                "control_name": "2309",
                "results": [
                    {
                        "instrument": "XT2000-OPEN",
                        "date": date,
                        "value": "3.94",
                        "code": "WBC",
                    },
                    {
                        "instrument": "XT2000-OPEN",
                        "date": date,
                        "value": "42.94",
                        "code": "RBC",
                    },
                    {
                        "instrument": "XT2000",
                        "date": date,
                        "value": "42",
                        "code": "PLT",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_no_legacy_mode_open_avec_analyseur(self):
    self.configure(legacy_mode_open=False)
    date = datetime.datetime(2013, 10, 4, 10, 25)
    result = self.decode(
        """
    H|^~\\&|||||||||||A.2|201310041026
    S|1|Manual|13644|||QC||||QC-23090801|WBC|3.94|||201310041025|
    S|1|Open|13644|||QC||||QC-23090801|RBC|42.94|||201310041025|
    S|1|Unknown|13644|||QC||||QC-23090801|PLT|42|||201310041025|
    S|27|Open|13644|||QC||||QC-23090801|H_INID|13644|||201310041025|
    S|28|Open|13644|||QC||||QC-23090801|H_INST|XT-1800i|||201310041025|
    L|1||0|41"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_level": "0801",
                "control_lot": "23090801",
                "control_name": "2309",
                "results": [
                    {
                        "instrument": "XT-1800i-OPEN",
                        "date": date,
                        "value": "3.94",
                        "code": "WBC",
                    },
                    {
                        "instrument": "XT-1800i-OPEN",
                        "date": date,
                        "value": "42.94",
                        "code": "RBC",
                    },
                    {
                        "instrument": "XT-1800i",
                        "date": date,
                        "value": "42",
                        "code": "PLT",
                    },
                ],
            }
        ],
    }
    assert result == expected
