# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

parser_class = "sysmex_suit_plus"


def test_result_rack(self):
    self.configure(sysmex_suitplus_format_rack=True)
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201806201441
    P|1|716750|||Tri hbg^VALIDATIONTOSOH||19850123|F||||||||0.0|0.0|||||||||||||||20180620
    OBR|1||04180022667|TS_COMP^TS_COMP|||||||||||^|ZZ^||ZZZ^|^|||201806201441|||||
    OBX|1|ST|TS_COMP^TS_COMP||ARCHIVE^||||||F^|201806201441|||TS-10^TS-10-1^000026^58|
    L|1||1|5"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "04180022667",
                "rack": "TS-10_000026-58",
                "type": "patient",
                "results": [
                    {
                        "instrument": "TS-10-TS-10-1",
                        "flags": [],
                        "value": "ARCHIVE",
                        "date": datetime.datetime(2018, 6, 20, 14, 41),
                        "code": "TS_COMP",
                        "lots": [],
                    },
                    {"code": "RACK", "value": "TS-10_000026-58"},
                ],
            }
        ],
    }
    assert result == expected


def test_result(self):
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201210011014
    P|1|Patient ID|||First Name^Last Name||19821001|M||||||||178.0|85.0|||||||OP|
    C|1|PATIENT^COMMENT|Free Patient Comment
    OBR|1||Sample ID|WBC|S||201210011006||||||||^Collection Source|Doctor^Doctor Name|Phone|Clinic^|Ward^|||201210011014||001|||
    C|1|INFO^RULE|Analyser error -> Please check IPU
    OBX|1|NM|WBC^WBC|D|6.10^|10*3/µL|3.90-10.20||NR^Z1234^Serial Number||F^Validate|201210011014|||XN-20^Serial Number^100125^02|Support
    C|1|HEMATOLOGY^COMMENT|Error_Func
    C|2|RBC^IP|Turbidity/HGB_Interference?
    C|3|WBC^GRAPH|PNG&R&20121001&R&2012_10_01_10_14_7c9289b9-b9ec-4ce9-8f97-666472897e20_WPC_BLAH.PNG
    C|4|PLT^GRAPH|PNG&R&20240220&R&2024_02_20_10_12_82ab8371-963f-412b-9a75-de6c2cbbbe7c_SCAT_PLT-O.PNG
    L|1||1|50"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "Sample ID",
                "type": "patient",
                "rack": "100125-02",
                "results": [
                    {
                        "flags": [],
                        "instrument": "XN-20-Serial Number",
                        "date": datetime.datetime(2012, 10, 1, 10, 14),
                        "value": "6.10",
                        "code": "WBC",
                        "lots": [{"name": "NR", "lot": "Z1234", "type": "reagent"}],
                    },
                    {
                        "code": "WPC_BLAH.PNG",
                        "value": "PNG/20121001/2012_10_01_10_14_7c9289b9-b9ec-4ce9-8f97-666472897e20_WPC_BLAH.PNG",
                        "instrument": "XN-20-Serial Number",
                    },
                    {
                        "code": "SCAT_PLT-O.PNG",
                        "value": "PNG/20240220/2024_02_20_10_12_82ab8371-963f-412b-9a75-de6c2cbbbe7c_SCAT_PLT-O.PNG",
                        "instrument": "XN-20-Serial Number",
                    },
                    {
                        "comments": ["Error_Func", "Turbidity/HGB_Interference?"],
                        "value": "",
                        "code": "ALARME",
                    },
                    {"code": "RACK", "value": "100125-02"},
                    {
                        "code": "INFO_RULE1",
                        "value": "Analyser error -> Please check IPU",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result_commentaires_multiples(self):
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201809191254
    P|1|228755|||Laura^Z_labo|TEST|19880508|F||||||||0.0|0.0|||||||||||||||20180919
    OBR|1||RESU4|BASO#^BASO#~BASO%^BASO%~EO#^EO#~EO%^EO%~IG#^IG#~IG%^IG%~LYMPH#^LYMPH#~LYMPH%^LYMPH%~MONO#^MONO#~MONO%^MONO%~NEUT#^NEUT#~NEUT%^NEUT%~W_WBCCONFIRM^W_WBCCONFIRM|||201809190947||||||||^|^||ZATIM^ZATIM|^|||201809191253||001|||
    C|1|WBC^MORPH|ASERA
    C|2|WBC^MORPH|AUER
    C|3|WBC^MORPH|AUERF
    C|4|RBC^MORPH|ACANT
    C|5|RBC^MORPH|ANIP
    C|6|RBC^MORPH|CPJ
    C|7|PLT^MORPH|Presence de macroplaquettes.
    C|8|PLT^MORPH|Presence de micromegacaryocytes.
    C|9|PLT^MORPH|Presence de plaquettes grises.
    C|10|BIOLO^MORPH|Frottis sanguin contrôlé par le biologiste
    OBX|1|NM|BASO#^BASO#|D|0.12^|10*3/µL|0.00-0.20||||F^Validate|201809191253|||XN-10^35629^^|aam3
    OBX|2|NM|BASO%^BASO%|D|1.2^|%|0.0-1.8||||F^Validate|201809191253|||XN-10^35629^^|aam3
    L|1||1|26"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "RESU4",
                "rack": None,
                "results": [
                    {
                        "instrument": "XN-10-35629",
                        "flags": [],
                        "value": "0.12",
                        "date": datetime.datetime(2018, 9, 19, 12, 53),
                        "code": "BASO#",
                        "lots": [],
                    },
                    {
                        "instrument": "XN-10-35629",
                        "flags": [],
                        "value": "1.2",
                        "date": datetime.datetime(2018, 9, 19, 12, 53),
                        "code": "BASO%",
                        "lots": [],
                    },
                    {"code": "WBC_MORPH1", "value": "ASERA"},
                    {"code": "WBC_MORPH2", "value": "AUER"},
                    {"code": "WBC_MORPH3", "value": "AUERF"},
                    {"code": "RBC_MORPH1", "value": "ACANT"},
                    {"code": "RBC_MORPH2", "value": "ANIP"},
                    {"code": "RBC_MORPH3", "value": "CPJ"},
                    {"code": "PLT_MORPH1", "value": "Presence de macroplaquettes."},
                    {"code": "PLT_MORPH2", "value": "Presence de micromegacaryocytes."},
                    {"code": "PLT_MORPH3", "value": "Presence de plaquettes grises."},
                    {
                        "code": "BIOLO_MORPH1",
                        "value": "Frottis sanguin contrôlé par le biologiste",
                    },
                    {"code": "WBCRBC_MORPH1", "value": "ASERA"},
                    {"code": "WBCRBC_MORPH2", "value": "AUER"},
                    {"code": "WBCRBC_MORPH3", "value": "AUERF"},
                    {"code": "WBCRBC_MORPH4", "value": "ACANT"},
                    {"code": "WBCRBC_MORPH5", "value": "ANIP"},
                    {"code": "WBCRBC_MORPH6", "value": "CPJ"},
                    {"code": "WBC_MORPH", "value": "ASERA;AUER;AUERF"},
                    {"code": "RBC_MORPH", "value": "ACANT;ANIP;CPJ"},
                    {
                        "code": "PLT_MORPH",
                        "value": "Presence de macroplaquettes.;Presence de micromegacaryocytes.;Presence de plaquettes grises.",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result_analyseur_id(self):
    self.configure(sysmex_instrument_id_seul=True)
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201210011014
    P|1|Patient ID|||First Name^Last Name||19821001|M||||||||178.0|85.0|||||||OP|
    C|1|PATIENT^COMMENT|Free Patient Comment
    OBR|1||Sample ID|WBC|S||201210011006||||||||^Collection Source|Doctor^Doctor Name|Phone|Clinic^|Ward^|||201210011014||001|||
    OBX|1|NM|WBC^WBC|D|6.10^|10*3/µL|3.90-10.20||NR^Z1234^Serial Number||F^Validate|201210011014|||XN-20^Serial Number^100125^02|Support
    L|1||1|50"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "Sample ID",
                "rack": "100125-02",
                "type": "patient",
                "results": [
                    {
                        "flags": [],
                        "instrument": "Serial Number",
                        "date": datetime.datetime(2012, 10, 1, 10, 14),
                        "lots": [{"name": "NR", "lot": "Z1234", "type": "reagent"}],
                        "value": "6.10",
                        "code": "WBC",
                    },
                    {"code": "RACK", "value": "100125-02"},
                ],
            }
        ],
    }
    assert result == expected


def test_result_no_rack(self):
    self.configure(analyseurs_a_ignorer="SP-10-24191")
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201709211818
    P|1|586565|||Archive paillasse tosoh^VALIDATION||19800101|M||||||||0.0|0.0|||||||||||||||20170921
    OBR|1||04180018647|SP^SP|S||||||||||^|BPEWEB^||TESTBPE^|^|||201709211818||001|||
    OBX|1|ST|SP^SP|D|STAIN:OK^||||||F^|201709211818|||SP-10^24191^000001^03|
    L|1||1|5
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "04180018647",
                "type": "patient",
                "rack": None,
                "results": [
                    {
                        "instrument": "SP-10-24191",
                        "flags": [],
                        "value": "STAIN:OK",
                        "date": datetime.datetime(2017, 9, 21, 18, 18),
                        "code": "SP",
                        "lots": [],
                    },
                    {"code": "RACK", "value": "000001-03"},
                ],
            }
        ],
    }
    assert result == expected


def test_resultat_obx_field_multiple(self):
    date = datetime.datetime(2019, 11, 11, 11, 31)
    result = self.decode(
        """
    H|^~\\&|||||||||||SUITplus1.0|201911111131
    P|1|88311|||TEST^ESSAI||19911204|F||||||||0.0|0.0|||||||||||||||20191111
    OBR|1||1230273V|RDW-SD^RDW-SD~RBC^RBC|||201911110848||||||||^|^||^|^|||201911111131||001|||
    OBX|26|NM|RDW-SD^RDW-SD|D|54.2^|fL|||||F^Validate|201911111131|||XN-10^28148^000042^7|KLEMA
    OBX|27|NM|RBC^RBC|D|4.58^ANISO~DACRYO~COMMENT1~COMMENT2|10*12/L|3.90-5.20||||F^Validate|201911111131|||XN-10^28148^000042^7|KLEMA
    L|1||1|37
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "1230273V",
                "type": "patient",
                "rack": "000042-7",
                "results": [
                    {
                        "code": "RDW-SD",
                        "date": date,
                        "flags": [],
                        "instrument": "XN-10-28148",
                        "lots": [],
                        "value": "54.2",
                    },
                    {
                        "code": "RBC",
                        "date": date,
                        "flags": [],
                        "instrument": "XN-10-28148",
                        "lots": [],
                        "value": "4.58",
                    },
                    {"code": "RACK", "value": "000042-7"},
                    {
                        "code": "RBC_COMMENT",
                        "date": date,
                        "instrument": "XN-10-28148",
                        "value": "ANISO;DACRYO;COMMENT1;COMMENT2",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_resultat_with_comment(self):
    date = datetime.datetime(2019, 4, 2, 11, 49)
    result = self.decode(
        """
    H|^~\\&|||||||||||SUITplus1.0|201904021149
    P|1|67404|||TEST^ANONYME||19271021|M||||||||0.0|0.0|||||||||||||||20190402
    OBR|1||1138323V|BASO#^BASO#~PDW^IDP~PLT^PLT|S||||||||||^|900082(2913)^900082(2913)||^|^|||201904021149||001|||
    OBX|1|NM|BASO#^BASO#|D|0.12^|10*9/L|0.00-0.20||||F^Validate|201904021149|||XN-10^28148^000032^2|JOVVI
    OBX|2|NM|PDW^IDP|D|11.8^|fL|||||F^Validate|201904021149|||XN-10^28148^000032^2|JOVVI
    OBX|3|NM|PLT^PLT|D|111^ABSAGPLT|10*9/L|160-370||||F^Validate|201904021149|||XN-10^28148^000032^2|JOVVI
    L|1||1|3
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "1138323V",
                "type": "patient",
                "rack": "000032-2",
                "results": [
                    {
                        "code": "BASO#",
                        "date": date,
                        "flags": [],
                        "instrument": "XN-10-28148",
                        "lots": [],
                        "value": "0.12",
                    },
                    {
                        "code": "PDW",
                        "date": date,
                        "flags": [],
                        "instrument": "XN-10-28148",
                        "lots": [],
                        "value": "11.8",
                    },
                    {
                        "code": "PLT",
                        "date": date,
                        "flags": [],
                        "instrument": "XN-10-28148",
                        "lots": [],
                        "value": "111",
                    },
                    {"code": "RACK", "value": "000032-2"},
                    {
                        "code": "PLT_COMMENT",
                        "value": "ABSAGPLT",
                        "date": date,
                        "instrument": "XN-10-28148",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_resu_comment(self):
    self.configure(traiter_comment_obx=True)
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201306041131
    P|3|00605642624|00605642624|00605642624|ANO^NYME||19490325|M
    OBR|1||04130124536||A||20130604||||L|||20130604||||||||20130604||001|F
    OBX|6|NM|HGB^HGB|D|7.5^XN_TEL|g/dL|||||F^Validate|202202231411|||XN-10^40876^333^3|0134410a
     L|1||4|56"""
    )
    assert result.samples[0].results[0].comments == ["XN_TEL"]


def test_resultat_option_formatage_rack(self):
    self.configure(sysmex_suitplus_format_rack=True)
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201710161306
    P|1|599447|||Connexion3^HEMATOXN||19850123|F||||||||0.0|0.0|||||||||||||||20171016
    OBR|1||04180019318|TS_COMP^TS_COMP|||||||||||^|ZZ^||TESTCTAV^|^|||201710161306|||||
    OBX|1|ST|TS_COMP^TS_COMP||ARCHIVE^||||||F^|201710161306|||TS-10^TS-10-1^000001^4|
    L|1||1|5
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "04180019318",
                "type": "patient",
                "rack": "TS-10_000001-4",
                "results": [
                    {
                        "instrument": "TS-10-TS-10-1",
                        "flags": [],
                        "value": "ARCHIVE",
                        "date": datetime.datetime(2017, 10, 16, 13, 6),
                        "code": "TS_COMP",
                        "lots": [],
                    },
                    {"code": "RACK", "value": "TS-10_000001-4"},
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    self.configure(sysmex_suitplus_qc_no_analyseur=False)
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201305221608
    S|1|Closed|XN10^12292|||QC||||30701102|BASO#|0.33|||201305221609
    L|0||1|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_lot": "30701102",
                "control_name": "3070",
                "control_level": "1102",
                "results": [
                    {
                        "code": "BASO#",
                        "value": "0.33",
                        "date": datetime.datetime(2013, 5, 22, 16, 9),
                        "instrument": "XN10-12292",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_with_option_qc_no_analyseur(self):
    self.configure(sysmex_suitplus_qc_no_analyseur=True)
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201305221608
    S|1|Closed|XN10^12292|||QC||||30701102|BASO#|0.33|||201305221609
    L|0||1|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_lot": "30701102",
                "control_name": "3070",
                "control_level": "1102",
                "results": [
                    {
                        "code": "BASO#",
                        "value": "0.33",
                        "date": datetime.datetime(2013, 5, 22, 16, 9),
                        "instrument": "XN10",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_analyseur_id_seul(self):
    self.configure(sysmex_suitplus_qc_no_analyseur=True, sysmex_instrument_id_seul=True)
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201305221608
    S|1|Closed|XN10^12292|||QC||||30701102|BASO#|0.33|||201305221609
    L|0||1|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "",
                "type": "control",
                "control_level": "1102",
                "control_lot": "30701102",
                "control_name": "3070",
                "results": [
                    {
                        "code": "BASO#",
                        "value": "0.33",
                        "date": datetime.datetime(2013, 5, 22, 16, 9),
                        "instrument": "XN10",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_morph(self):
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201305231154
    P|1|02052100417527|||ANO^NYME|BELLSOLA|19460417|F||||||||0.0|0.0|||||||||||||||20130523
    OBR|1||12115700||||||0||||||^|371048273^||^|^|||201305231154||001|||
    C|1|WBC^MORPH|AM^Présence de Tricholeucocytes
    C|10|RBC^MORPH|AH^Poïkilocytose
    C|18|PLT^MORPH|AW^Absence d'agrégats plaquettaires
    C|19|BIOLO^MORPH|Frottis sanguin contrôlé par le biologiste
    OBX|1|NM|W_SNE#^SEG#|D|3.58^|10*3/µL|||||F^|201305231154|||DM^CellaVision^^|
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "12115700",
                "type": "patient",
                "rack": None,
                "results": [
                    {
                        "flags": [],
                        "lots": [],
                        "instrument": "DM-CellaVision",
                        "date": datetime.datetime(2013, 5, 23, 11, 54),
                        "value": "3.58",
                        "code": "W_SNE#",
                    },
                    {"value": "AM", "code": "WBC_MORPH1"},
                    {"value": "AH", "code": "RBC_MORPH1"},
                    {"value": "AW", "code": "PLT_MORPH1"},
                    {
                        "value": "Frottis sanguin contrôlé par le biologiste",
                        "code": "BIOLO_MORPH1",
                    },
                    {"value": "AM", "code": "WBCRBC_MORPH1"},
                    {"value": "AH", "code": "WBCRBC_MORPH2"},
                    {"value": "AM", "code": "WBC_MORPH"},
                    {"value": "AH", "code": "RBC_MORPH"},
                    {"value": "AW", "code": "PLT_MORPH"},
                ],
            }
        ],
    }
    assert result == expected


def test_alarme_optique(self):
    result = self.decode(
        """H|^~\\&|||||||||||SUITplus1.0|201305231154
    P|1|02052100417527|||ANO^NYME|BELLSOLA|19460417|F
    OBR|1||12115700||||||0||||||^|371048273^||^|^|||201305231154||001
    OBX|28|NM|PLT^PLT|D|57^&|10*3/µL|160-370|W|||F^|201305311603
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "12115700",
                "type": "patient",
                "rack": None,
                "results": [
                    {
                        "lots": [],
                        "instrument": "-",
                        "code": "PLT",
                        "value": "57",
                        "date": datetime.datetime(2013, 5, 31, 16, 3),
                        "flags": ["W", "O"],
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|^~\\&|||||||||||SUITplus1.0|201307151419
        P|1|036666666|036666666||ANO^NYME|NJF|20110513|F|||||ATTEND^LAGRANGE Joseph-Loui||||||||||||SER
        OBR|1|123456||A~B|S||201305131142||||L||Commentaire patient no1|||ATTEND^LAGRANGE Joseph-Louis||SER|SER^SERVICE
        OBR|2|123456||C|S||201305131142||||C||Commentaire patient no1|||ATTEND^LAGRANGE Joseph-Louis||SER|SER^SERVICE
        L|1||1|5"""
    assert result == expected
