# -*- coding: utf8 -*-
from __future__ import unicode_literals
import copy

from driverng.tests import BaseTest


class TestVitek(BaseTest):
    MAPPING_CLASS = "vitek"

    def test_resultats_atb_adagio(self):
        self.F.prefixe_atb_adagio = "B"
        self.assertResultEqual(
            """
        mtrsl|iiV2|pi134509|pnTEST_ENSEMENCEMENT BUCCAL|si|s128/02/2019|ssBUC|st|slnom du|ci4001LE90228002|ctBUC|rtADAGIO|t11|o1mormor|a1mec|a4R|a1fos|a4S|zz|""",
            {
                "4001LE90228002": {
                    "resultats": [
                        {
                            "atb": [
                                (("ANTIBIO", "B_MEC"), ("CMI", ""), ("SENSI", "R")),
                                (("ANTIBIO", "B_FOS"), ("CMI", ""), ("SENSI", "S")),
                            ],
                            "date": None,
                            "nom_antibiogramme": "ADAGIO",
                            "resu": "G_MORMOR",
                            "seq": "1",
                            "test": "BUC",
                        }
                    ]
                }
            },
        )

    def test_resultats(self):
        self.assertResultEqual(
            """
        mtrsl|pi03042250173|si|ci520000636208|c011290|ctURET|cnURET|ta|rtAST-P606|t11|o1straga|ra|a1peng|a3<=0,12|a4S|ra|a1am|a3<=0,25|a4S|ra|a1lev|a31|a4S|ra|a1mxf|a3<=0,25|a4S|ra|a1e|a3>=8|a4R|ra|a1cc|a3>=8|a4R|ra|a1lnz|a32|a4S|ra|a1tpn|a3<=0,5|a4S|ra|a1va|a3<=0,5|a4S|ra|a1tet|a3>=16|a4R|ra|a1ftn|a3<=16|a4S|ra|a1c|a3<=4|a4S|ra|a1sxt|a3<=10|a4S|zz""",
            {
                "520000636208": {
                    "resultats": [
                        {
                            "atb": [
                                (
                                    ("ANTIBIO", "A_PENG"),
                                    ("CMI", "<=0,12"),
                                    ("SENSI", "S"),
                                ),
                                (
                                    ("ANTIBIO", "A_AM"),
                                    ("CMI", "<=0,25"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_LEV"), ("CMI", "1"), ("SENSI", "S")),
                                (
                                    ("ANTIBIO", "A_MXF"),
                                    ("CMI", "<=0,25"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_E"), ("CMI", ">=8"), ("SENSI", "R")),
                                (("ANTIBIO", "A_CC"), ("CMI", ">=8"), ("SENSI", "R")),
                                (("ANTIBIO", "A_LNZ"), ("CMI", "2"), ("SENSI", "S")),
                                (
                                    ("ANTIBIO", "A_TPN"),
                                    ("CMI", "<=0,5"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_VA"), ("CMI", "<=0,5"), ("SENSI", "S")),
                                (("ANTIBIO", "A_TET"), ("CMI", ">=16"), ("SENSI", "R")),
                                (("ANTIBIO", "A_FTN"), ("CMI", "<=16"), ("SENSI", "S")),
                                (("ANTIBIO", "A_C"), ("CMI", "<=4"), ("SENSI", "S")),
                                (("ANTIBIO", "A_SXT"), ("CMI", "<=10"), ("SENSI", "S")),
                            ],
                            "date": None,
                            "nom_antibiogramme": "AST-P606",
                            "resu": "G_STRAGA",
                            "seq": "1",
                            "test": "URET",
                        }
                    ]
                }
            },
        )

    def test_results_with_phenotype(self):
        self.assertResultEqual(
            """mtrsl|iiV2|pi443335|pn|si|ci1324091899|ctURINE|ta|rtAST-P631|t11|o1staaur|afBÊTA-LACTAMINES|apPÉNICILLINASE ACQUISE|apPÉNICILLINASE ACQ OU MODIFICATION DES PLP|afTÉTRACYCLINES|apSAUVAGE|afACIDE FUSIDIQUE|apSAUVAGE|afFOSFOMYCINE|apRÉSISTANT|afAMINOSIDES|apSAUVAGE|afGLYCOPEPTIDES|apSAUVAGE|afRIFAMYCINES|apSAUVAGE|afQUINOLONES|apRÉSISTANT|apRÉSISTANCE PARTIELLE|afTRIMÉTHOPRIME/SULFAMIDES|apSAUVAGE|apRÉSISTANT AU TRIMETHOPRIME|afFURANES|apSAUVAGE|afOXAZOLIDINONE|apSAUVAGE|afMACROLIDES/LINCOSAMIDES/STREPTOGRAMINES|apSAUVAGE|ra|a1peng|a3>0,25|a4R|ra|a1ox|a31|a4S|ra|a1gm|a3<=0,5|a4S|ra|a1k|a3<=4|a4S|ra|a1tob|a3<=1|a4S|ra|a1ofx|a3>4|a4R|ra|a1lnz|a32|a4S|ra|a1tpn|a3<=0,5|a4S|ra|a1va|a3<=0,5|a4S|ra|a1tet|a3<=1|a4S|ra|a1fos|a364|a4R|ra|a1ftn|a3<=16|a4S|ra|a1rif|a3<=0,03|a4S|ra|a1sxt|a3<=10|a4S|zz|
            """,
            {
                "1324091899": {
                    "rack": None,
                    "resultats": [
                        {
                            "atb": [
                                (
                                    ("ANTIBIO", "A_PENG"),
                                    ("CMI", ">0,25"),
                                    ("SENSI", "R"),
                                ),
                                (("ANTIBIO", "A_OX"), ("CMI", "1"), ("SENSI", "S")),
                                (("ANTIBIO", "A_GM"), ("CMI", "<=0,5"), ("SENSI", "S")),
                                (("ANTIBIO", "A_K"), ("CMI", "<=4"), ("SENSI", "S")),
                                (("ANTIBIO", "A_TOB"), ("CMI", "<=1"), ("SENSI", "S")),
                                (("ANTIBIO", "A_OFX"), ("CMI", ">4"), ("SENSI", "R")),
                                (("ANTIBIO", "A_LNZ"), ("CMI", "2"), ("SENSI", "S")),
                                (
                                    ("ANTIBIO", "A_TPN"),
                                    ("CMI", "<=0,5"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_VA"), ("CMI", "<=0,5"), ("SENSI", "S")),
                                (("ANTIBIO", "A_TET"), ("CMI", "<=1"), ("SENSI", "S")),
                                (("ANTIBIO", "A_FOS"), ("CMI", "64"), ("SENSI", "R")),
                                (("ANTIBIO", "A_FTN"), ("CMI", "<=16"), ("SENSI", "S")),
                                (
                                    ("ANTIBIO", "A_RIF"),
                                    ("CMI", "<=0,03"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_SXT"), ("CMI", "<=10"), ("SENSI", "S")),
                            ],
                            "seq": "1",
                            "test": "URINE",
                            "date": None,
                            "resu": "G_STAAUR",
                            "nom_antibiogramme": "AST-P631",
                            "phenotypes": [
                                "BÊTA-LACTAMINES-PÉNICILLINASE ACQUISE",
                                "BÊTA-LACTAMINES-PÉNICILLINASE ACQ OU MODIFICATION DES PLP",
                                "TÉTRACYCLINES-SAUVAGE",
                                "ACIDE FUSIDIQUE-SAUVAGE",
                                "FOSFOMYCINE-RÉSISTANT",
                                "AMINOSIDES-SAUVAGE",
                                "GLYCOPEPTIDES-SAUVAGE",
                                "RIFAMYCINES-SAUVAGE",
                                "QUINOLONES-RÉSISTANT",
                                "QUINOLONES-RÉSISTANCE PARTIELLE",
                                "TRIMÉTHOPRIME/SULFAMIDES-SAUVAGE",
                                "TRIMÉTHOPRIME/SULFAMIDES-RÉSISTANT AU TRIMETHOPRIME",
                                "FURANES-SAUVAGE",
                                "OXAZOLIDINONE-SAUVAGE",
                                "MACROLIDES/LINCOSAMIDES/STREPTOGRAMINES-SAUVAGE",
                            ],
                        }
                    ],
                }
            },
        )

    def test_resultats_non_testes(self):
        self.assertResultEqual(
            """
        mtrsl|pi03091400110510|si|ci375091400587|ctHEMOC|rtAST-P606|t11|o1strfae|a1am|a3<=2|a4S|a1lev|a31|a4S|a1mxf|a3(-)|a4(-)|a1e|a34|a4R|a1cc|a3(-)|a4(-)|a1tet|a3(-)|a4(-)|a1ftn|a3<=16|a4S|a1c|a38|a4S|zz|""",
            {
                "375091400587": {
                    "resultats": [
                        {
                            "atb": [
                                (("ANTIBIO", "A_AM"), ("CMI", "<=2"), ("SENSI", "S")),
                                (("ANTIBIO", "A_LEV"), ("CMI", "1"), ("SENSI", "S")),
                                (("ANTIBIO", "A_E"), ("CMI", "4"), ("SENSI", "R")),
                                (("ANTIBIO", "A_FTN"), ("CMI", "<=16"), ("SENSI", "S")),
                                (("ANTIBIO", "A_C"), ("CMI", "8"), ("SENSI", "S")),
                            ],
                            "date": None,
                            "nom_antibiogramme": "AST-P606",
                            "resu": "G_STRFAE",
                            "seq": "1",
                            "test": "HEMOC",
                        }
                    ]
                }
            },
        )

    def test_resultats_ident(self):
        self.assertResultEqual(
            """mtrsl|pi03121700014174|si|ci344112718265|ctURINE|rtGN|t11|o1citdiv|o995|zz|""",
            {
                "344112718265": {
                    "resultats": [
                        {
                            "atb": [],
                            "date": None,
                            "nom_antibiogramme": "GN",
                            "resu": {
                                "type_resultat": "identification_germes",
                                "germes": [
                                    {"id": "G_CITDIV", "score": "95", "rang": 1}
                                ],
                            },
                            "seq": "0",
                            "test": "IDENT_URINE",
                        },
                        {"test": "GERME_1_URINE", "resu": "G_CITDIV"},
                    ]
                }
            },
        )

    def test_resultats_ident_cplx(self):
        self.assertResultEqual(
            """mtrsl|pi03121700014174|si|ci344112718265|ctURINE|rtGN|t11|o1citdiv|zz|""",
            {
                "344112718265": {
                    "resultats": [
                        {
                            "atb": [],
                            "date": None,
                            "nom_antibiogramme": "GN",
                            "resu": {
                                "type_resultat": "identification_germes",
                                "germes": [{"id": "G_CITDIV", "rang": 1}],
                            },
                            "seq": "0",
                            "test": "IDENT_URINE",
                        },
                        {"test": "GERME_1_URINE", "resu": "G_CITDIV"},
                    ]
                }
            },
        )

    def test_resultats_strip(self):
        self.assertResultEqual(
            """mtrsl|pi03121700014174|si|ci344112718265|ctURINE|rtGN|t11|o1citdiv|zz|
            """,
            {
                "344112718265": {
                    "resultats": [
                        {
                            "atb": [],
                            "date": None,
                            "nom_antibiogramme": "GN",
                            "resu": {
                                "type_resultat": "identification_germes",
                                "germes": [{"id": "G_CITDIV", "rang": 1}],
                            },
                            "seq": "0",
                            "test": "IDENT_URINE",
                        },
                        {"test": "GERME_1_URINE", "resu": "G_CITDIV"},
                    ]
                }
            },
            unindent=False,
        )

    def test_results_comments(self):
        self.assertResultEqual(
            """
        mtrsl|iiV2|is000012C76751|itSU|pipatient test G3|pnpatient test G3, patient test G3|si|s034447|s105/02/2024|s305/02/2024|ci3467357ECBU|c034447|ctECBU|cnECBU|ta|rtAST-N372|rr4944955|tuAST-N372|tg0222722104|te10/05/2025|ncC3GGP3|ndC3GGP3|t11|o1entaer|o2Enterobacter aerogenes|acConcordant|afBÃTA-LACTAMINES|apSAUVAGE (CÃPHALOSPORINASE)|afAMINOSIDES|apSAUVAGE|afQUINOLONES|apSAUVAGE|afTRIMÃTHOPRIME/SULFAMIDES|apSAUVAGE|apRÃSISTANT AU TRIMETHOPRIME|ra|a1tem|a2TÃ©mocilline|a3<=4|a4S|anS|ra|a1amc|a2Amoxicilline/acide clavulanique|a3>=32|a4R|anR|ra|a1tic|a2Ticarcilline|a3<=8|a4S|anS|ra|a1fox|a2CÃ©foxitine|a3>=64|a4R|anR|ra|a1cfm|a2CÃ©fixime|a32|a4R|anR|ra|a1taz|a2Ceftazidime|a3<=1|a4S|anS|ra|a1ctr|a2Ceftriaxone|a3<=1|a4S|anS|ra|a1etp|a2ErtapÃ©nÃšme|a3<=0,12|a4S|anS|ra|a1an|a2Amikacine|a3<=2|a4S|anS|ra|a1gm|a2Gentamicine|a3<=1|a4S|anS|ra|a1na|a2Acide nalidixique|a34|a4S|anS|ra|a1ofx|a2Ofloxacine|a3<=0,25|a4S|anS|ra|a1sxt|a2TrimÃ©thoprime/sulfamÃ©thoxazole|a3<=20|a4S|anS|zz|""",
            {
                "3467357ECBU": {
                    "rack": None,
                    "resultats": [
                        {
                            "atb": [
                                (("ANTIBIO", "A_TEM"), ("CMI", "<=4"), ("SENSI", "S")),
                                (("ANTIBIO", "A_AMC"), ("CMI", ">=32"), ("SENSI", "R")),
                                (("ANTIBIO", "A_TIC"), ("CMI", "<=8"), ("SENSI", "S")),
                                (("ANTIBIO", "A_FOX"), ("CMI", ">=64"), ("SENSI", "R")),
                                (("ANTIBIO", "A_CFM"), ("CMI", "2"), ("SENSI", "R")),
                                (("ANTIBIO", "A_TAZ"), ("CMI", "<=1"), ("SENSI", "S")),
                                (("ANTIBIO", "A_CTR"), ("CMI", "<=1"), ("SENSI", "S")),
                                (
                                    ("ANTIBIO", "A_ETP"),
                                    ("CMI", "<=0,12"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_AN"), ("CMI", "<=2"), ("SENSI", "S")),
                                (("ANTIBIO", "A_GM"), ("CMI", "<=1"), ("SENSI", "S")),
                                (("ANTIBIO", "A_NA"), ("CMI", "4"), ("SENSI", "S")),
                                (
                                    ("ANTIBIO", "A_OFX"),
                                    ("CMI", "<=0,25"),
                                    ("SENSI", "S"),
                                ),
                                (("ANTIBIO", "A_SXT"), ("CMI", "<=20"), ("SENSI", "S")),
                            ],
                            "seq": "1",
                            "test": "ECBU",
                            "date": None,
                            "resu": "G_ENTAER",
                            "nom_antibiogramme": "AST-N372",
                            "commentaires": ["C3GGP3"],
                            "phenotypes": [
                                "BÃ\x8aTA-LACTAMINES-SAUVAGE (CÃ\x89PHALOSPORINASE)",
                                "AMINOSIDES-SAUVAGE",
                                "QUINOLONES-SAUVAGE",
                                "TRIMÃ\x89THOPRIME/SULFAMIDES-SAUVAGE",
                                "TRIMÃ\x89THOPRIME/SULFAMIDES-RÃ\x89SISTANT AU TRIMETHOPRIME",
                            ],
                        }
                    ],
                }
            },
        )

    def test_order(self):
        self.assertOrderMessageEqual(
            self.WORKLIST_NEW_ADD_DEL,
            "mtmpr|pi036666666|pnNYME_ANO|pb13/05/2011|psF|plSER|p2SERNAME|pxSER|p3SERNAME|ppPRS|p5LAGRANGE|pa11/05/2013|si|ssA|s5A|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctA|cnA|si|ssB|s5B|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctB|cnB|si|ssC|s5C|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctC|cnC|zz",
        )

    def test_order_with_option(self):
        self.F.envoi_germe = True
        self.F.vitek_lower_germe = True
        worklist = copy.deepcopy(self.WORKLIST_NEW_ADD_DEL)
        worklist.samples[0]["results"][0]["resu_json"]["germes"][0]["Id"] = "G_IDENT"
        expected = """mtmpr|pi036666666|pnNYME_ANO|pb13/05/2011|psF|plSER|p2SERNAME|pxSER|p3SERNAME|ppPRS|p5LAGRANGE|pa11/05/2013|si|ssA|s5A|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctA|cnA|ta|o1ident|zz
        mtmpr|pi036666666|pnNYME_ANO|pb13/05/2011|psF|plSER|p2SERNAME|pxSER|p3SERNAME|ppPRS|p5LAGRANGE|pa11/05/2013|si|ssA|s5A|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctA|cnA|ta|o12|zz
        mtmpr|pi036666666|pnNYME_ANO|pb13/05/2011|psF|plSER|p2SERNAME|pxSER|p3SERNAME|ppPRS|p5LAGRANGE|pa11/05/2013|si|ssA|s5A|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctA|cnA|si|ssB|s5B|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctB|cnB|si|ssC|s5C|spPRS|s8LAGRANGE|slSER|sxSER|s113/05/2013|s211:42|ci123456|ctC|cnC|zz
        """
        self.assertOrderMessageEqual(worklist, expected)
